using Jypeli;

namespace TippuKivet
{
    public class TippuKivet : PhysicsGame
    {
        /// <summary>
        /// Pelaajan liikkumisnopeudet oikealle ja vasemmalle.
        /// </summary>
        private readonly Vector nopeusOikea = new Vector(800, 0);
        private readonly Vector nopeusVasen = new Vector(-800, 0);

        /// <summary>
        /// Pelin keskeiset muuttujat.
        /// </summary>
        private PhysicsObject pelaaja;
        private IntMeter pistelaskuri;
        private Timer ajastinTimantit;
        private Timer ajastinKivet;
        private int ohimenneetTimantit;

        /// <summary>
        /// Pelin aloitusmetodi. Alustaa kentän, pelaajan ja ajastimet.
        /// </summary>
        public override void Begin()
        {
            SetWindowSize(800, 600, false);

            ajastinTimantit = Timer.CreateAndStart(1.5, LuoTimantti);
            ajastinKivet = Timer.CreateAndStart(3.0, LuoKivi);

            pistelaskuri = new IntMeter(0);
            LuoPisteNaytto();

            Gravity = new Vector(0, -700);

            pelaaja = LuoPelaaja(0);
            LuoKenttä();

            AsetaOhjaimet();
        }

        /// <summary>
        /// Luo pelialueen, alalaatan ja taustan.
        /// </summary>
        private void LuoKenttä()
        {
            PhysicsObject alalaatta = Level.CreateBottomBorder();
            alalaatta.IsVisible = false;
            AddCollisionHandler(alalaatta, "Kivi", KiviAlalaattaCollide);
            AddCollisionHandler(alalaatta, "Timantti", TimanttiAlalaattaCollide);

            Level.Background.Image = LoadImage("taustakuva");
            Level.Background.Color = Color.Black;

            Camera.ZoomToLevel();
        }

        /// <summary>
        /// Luo pelaaja-olion peliin.
        /// </summary>
        /// <param name="x">Pelaajan aloituspaikka x-koordinaatilla.</param>
        /// <returns>Palauttaa pelaaja-olion.</returns>
        private PhysicsObject LuoPelaaja(double x)
        {
            PhysicsObject karry = LuoObjekti(150, 200, x, -300, Color.Red, "Pelaaja");
            karry.IgnoresCollisionResponse = true;
            karry.IgnoresGravity = true;
            karry.Image = LoadImage("pelaaja");

            AddCollisionHandler(karry, "Kivi", PelaajaKiviCollide);
            AddCollisionHandler(karry, "Timantti", PelaajaTimanttiCollide);
            Add(karry);
            return karry;
        }

        /// <summary>
        /// Luo pistelaskurin näkymän ruudulle.
        /// </summary>
        private void LuoPisteNaytto()
        {
            Label pisteNaytto = new Label
            {
                Position = new Vector(340, 250),
                TextColor = Color.Red,
                Color = Color.White,
                Size = new Vector(5000, 5000),
                Title = "Pisteet: ",
            };
            pisteNaytto.BindTo(pistelaskuri);
            Add(pisteNaytto);
        }

        /// <summary>
        /// Luo satunnaiseen paikkaan ilmestyvän timantin.
        /// </summary>
        private void LuoTimantti()
        {
            double x = RandomGen.NextDouble(Level.Left + 50, Level.Right - 50);
            PhysicsObject timantti = LuoObjekti(60, 60, x, Level.Top + 50, Color.Yellow, "Timantti");
            timantti.AngularVelocity = RandomGen.NextDouble(-5.0, 5.0);
            timantti.Image = LoadImage("timantti.png");
            Add(timantti);
        }

        /// <summary>
        /// Luo satunnaiseen paikkaan ilmestyvän kiven.
        /// </summary>
        private void LuoKivi()
        {
            double x = RandomGen.NextDouble(Level.Left + 20, Level.Right - 20);
            double randomSize = RandomGen.NextDouble(30.0, 120.0);
            PhysicsObject kivi = LuoObjekti(randomSize, randomSize, x, Level.Top + 50, Color.White, "Kivi");
            kivi.Velocity = new Vector(0, -400);
            kivi.Image = LoadImage("kivi.png");
            Add(kivi);
        }

        /// <summary>
        /// Luo uuden peliobjektin annetulla koolla, sijainnilla ja värillä.
        /// </summary>
        /// <param name="width">Olion leveys.</param>
        /// <param name="height">Olion korkeus.</param>
        /// <param name="x">Sijainti x-koordinaatilla.</param>
        /// <param name="y">Sijainti y-koordinaatilla.</param>
        /// <param name="color">Olion väri.</param>
        /// <param name="tag">Olion tunniste (Tag).</param>
        /// <returns>Palauttaa luodun PhysicsObjectin.</returns>
        private PhysicsObject LuoObjekti(double width, double height, double x, double y, Color color, string tag)
        {
            return new PhysicsObject(width, height, Shape.Rectangle)
            {
                X = x,
                Y = y,
                Color = color,
                Tag = tag
            };
        }

        /// <summary>
        /// Pelaajan ja timantin törmäyskäsittelijä.
        /// </summary>
        /// <param name="_">Pelaaja (ei käytetä).</param>
        /// <param name="timantti">Törmännyt timantti.</param>
        private void PelaajaTimanttiCollide(PhysicsObject _, PhysicsObject timantti)
        {
            pistelaskuri.Value += 1;
            timantti.Destroy();
            KutistaPelaaja();

            if (pistelaskuri.Value >= 10)
            {
                VoitaPeli();
            }
        }

        /// <summary>
        /// Pelaajan ja kiven törmäyskäsittelijä.
        /// </summary>
        /// <param name="_">Pelaaja (ei käytetä).</param>
        /// <param name="kivi">Törmännyt kivi.</param>
        private void PelaajaKiviCollide(PhysicsObject _, PhysicsObject kivi)
        {
            kivi.Destroy();
            LopetaPeli("Kivi osui sinua päähän, Peli ohi!");
        }

        /// <summary>
        /// Kiven ja alalaatan törmäyskäsittelijä.
        /// </summary>
        /// <param name="_">Alalaatta (ei käytetä).</param>
        /// <param name="kivi">Putoava kivi.</param>
        private void KiviAlalaattaCollide(PhysicsObject _, PhysicsObject kivi)
        {
            kivi.Destroy();
        }

        /// <summary>
        /// Timantin ja alalaatan törmäyskäsittelijä.
        /// </summary>
        /// <param name="_">Alalaatta (ei käytetä).</param>
        /// <param name="timantti">Putoava timantti.</param>
        private void TimanttiAlalaattaCollide(PhysicsObject _, PhysicsObject timantti)
        {
            timantti.Destroy();
            ohimenneetTimantit++;

            if (ohimenneetTimantit >= 2)
            {
                LopetaPeli("Et ollut tarpeeksi tehokas!");
            }
        }

        /// <summary>
        /// Pienentää pelaajaa timantin keräämisen jälkeen.
        /// </summary>
        private void KutistaPelaaja()
        {
            const double shrinkFactor = 0.9;
            const double minSize = 10.0;

            if (pelaaja.Width > minSize && pelaaja.Height > minSize)
            {
                pelaaja.Width *= shrinkFactor;
                pelaaja.Height *= shrinkFactor;
            }
        }

        /// <summary>
        /// Suoritetaan, kun pelaaja voittaa pelin.
        /// </summary>
        private void VoitaPeli()
        {
            MessageDisplay.Add("Olet TippuKiviLuolan suvereeni Päällikkö!");
            ajastinTimantit.Stop();
            ajastinKivet.Stop();
            Timer.SingleShot(6, Exit);
            Pause();
        }

        /// <summary>
        /// Pysäyttää pelin ja näyttää häviöviestin.
        /// </summary>
        /// <param name="viesti">Näytettävä viesti pelaajalle.</param>
        private void LopetaPeli(string viesti)
        {
            MessageDisplay.Add(viesti);
            ajastinTimantit.Stop();
            ajastinKivet.Stop();
            Timer.SingleShot(7, Exit);
            Pause();
        }

        /// <summary>
        /// Asettaa pelin ohjaimet (liikkuminen, pause, sulkeminen).
        /// </summary>
        private void AsetaOhjaimet()
        {
            Keyboard.Listen(Key.Left, ButtonState.Down, AsetaNopeus, "Liikuta pelaajaa vasemmalle", pelaaja, nopeusVasen);
            Keyboard.Listen(Key.Left, ButtonState.Released, AsetaNopeus, null, pelaaja, Vector.Zero);
            Keyboard.Listen(Key.Right, ButtonState.Down, AsetaNopeus, "Liikuta pelaajaa oikealle", pelaaja, nopeusOikea);
            Keyboard.Listen(Key.Right, ButtonState.Released, AsetaNopeus, null, pelaaja, Vector.Zero);
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
            Keyboard.Listen(Key.P, ButtonState.Pressed, Pause, "Pysäyttää pelin");
        }

        /// <summary>
        /// Asettaa pelaajan nopeuden ja estää kentän reunojen ylittämisen.
        /// </summary>
        /// <param name="karry">Pelaaja, jonka nopeus asetetaan.</param>
        /// <param name="nopeus">Haluttu nopeus vektorina.</param>
        private void AsetaNopeus(PhysicsObject karry, Vector nopeus)
        {
            if ((pelaaja.X + pelaaja.Width / 2 >= Level.Right && nopeus.X > 0) ||
                (pelaaja.X - pelaaja.Width / 2 <= Level.Left && nopeus.X < 0))
            {
                pelaaja.Velocity = Vector.Zero;
            }
            else
            {
                pelaaja.Velocity = nopeus;
            }
        }
    }
}
